import {NgModule} from "@angular/core";
import {RouterModule} from "@angular/router";
import {PageNotFoundPresentational} from "@app/components/presentationals/page-not-found/page-not-found.presentational";
import {ApplicationRolePermissionEnum} from "@app/shared/enums/application-role-permission.enum";
import {AppRoutesEnum} from "@app/shared/enums/routes.enum";
import {ApplicationRoleGuardService} from "@app/shared/guards/application-role-guard.service";
import {DlcmRoutes} from "@app/shared/models/dlcm-route.model";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {ApplicationMaintenanceGuardService} from "@shared/guards/application-maintenance-guard.service";
import {
  DevGuardService,
  IconAppSummaryRoutable,
  MaintenanceModeRoutable,
  PreventLeaveGuardService,
  ServerOfflineModeRoutable,
  UrlQueryParamHelper,
} from "solidify-frontend";

const routes: DlcmRoutes = [
  {
    path: AppRoutesEnum.home,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/home/home.module").then(m => m.HomeModule),
    data: {
      permission: ApplicationRolePermissionEnum.noPermission,
    },
    canActivate: [ApplicationMaintenanceGuardService, ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.deposit,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/deposit/deposit.module").then(m => m.DepositModule),
    data: {
      breadcrumb: LabelTranslateEnum.deposit,
      permission: ApplicationRolePermissionEnum.userPermission,
    },
    canActivate: [ApplicationMaintenanceGuardService, ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.preservationSpace,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/preservation-space/preservation-space.module").then(m => m.PreservationSpaceModule),
    data: {
      breadcrumb: LabelTranslateEnum.preservationSpace,
      permission: ApplicationRolePermissionEnum.userPermission,
    },
    canActivate: [ApplicationMaintenanceGuardService, ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.admin,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/admin/admin.module").then(m => m.AdminModule),
    data: {
      breadcrumb: LabelTranslateEnum.administration,
      permission: ApplicationRolePermissionEnum.adminPermission,
    },
    canActivate: [ApplicationMaintenanceGuardService, ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.preservationPlanning,
    // @ts-ignore Dynamic import
    loadChildren: () => import("@preservation-planning/preservation-planning.module").then(m => m.PreservationPlanningModule),
    data: {
      breadcrumb: LabelTranslateEnum.preservationPlanning,
      permission: ApplicationRolePermissionEnum.adminPermission,
    },
    canActivate: [ApplicationMaintenanceGuardService, ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.order,
    // @ts-ignore Dynamic import
    loadChildren: () => import("./features/order/order.module").then(m => m.OrderModule),
    data: {
      breadcrumb: LabelTranslateEnum.archiveOrders,
      permission: ApplicationRolePermissionEnum.userPermission,
    },
    canActivate: [ApplicationMaintenanceGuardService, ApplicationRoleGuardService],
  },
  {
    path: AppRoutesEnum.icons,
    component: IconAppSummaryRoutable,
    canActivate: [DevGuardService],
  },
  {
    path: AppRoutesEnum.maintenance,
    component: MaintenanceModeRoutable,
    canDeactivate: [PreventLeaveGuardService],
  },
  {
    path: AppRoutesEnum.serverOffline,
    component: ServerOfflineModeRoutable,
    canDeactivate: [PreventLeaveGuardService],
  },
  {
    path: AppRoutesEnum.root,
    redirectTo: "/" + AppRoutesEnum.home + UrlQueryParamHelper.getQueryParamOAuth2(),
    pathMatch: "full",
  },
  {
    path: AppRoutesEnum.index,
    redirectTo: "/" + AppRoutesEnum.home + UrlQueryParamHelper.getQueryParamOAuth2(),
    pathMatch: "full",
  },
  {
    path: "**",
    component: PageNotFoundPresentational,
  },
];

@NgModule({
  imports: [RouterModule.forRoot(routes, {
    useHash: false,
    relativeLinkResolution: "legacy",
  })],
  exports: [RouterModule],
})
export class AppRoutingModule {
}
