import {
  AdminIndexFieldAliasAction,
  adminIndexFieldAliasActionNameSpace,
} from "@admin/index-field-alias/stores/admin-index-field-alias.action";
import {Injectable} from "@angular/core";
import {AppSystemPropertyAction} from "@app/stores/system-property/app-system-property.action";
import {AppSystemPropertyStateModel} from "@app/stores/system-property/app-system-property.state";
import {IndexFieldAlias} from "@models";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiEnum} from "@shared/enums/api.enum";
import {
  RoutesEnum,
  urlSeparator,
} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {Observable} from "rxjs";
import {
  ApiService,
  defaultResourceStateInitValue,
  isNullOrUndefined,
  MARK_AS_TRANSLATABLE,
  NotificationService,
  ResourceState,
  ResourceStateModel,
  StoreUtil,
} from "solidify-frontend";

export interface AdminIndexFieldAliasStateModel extends ResourceStateModel<IndexFieldAlias> {
}

@Injectable()
@State<AdminIndexFieldAliasStateModel>({
  name: StateEnum.admin_indexFieldAlias,
  defaults: {
    ...defaultResourceStateInitValue(),
  },
  children: [],
})
export class AdminIndexFieldAliasState extends ResourceState<AdminIndexFieldAliasStateModel, IndexFieldAlias> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: adminIndexFieldAliasActionNameSpace,
      routeRedirectUrlAfterSuccessCreateAction: (resId: string) => RoutesEnum.adminIndexFieldAliasDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessUpdateAction: (resId: string) => RoutesEnum.adminIndexFieldAliasDetail + urlSeparator + resId,
      routeRedirectUrlAfterSuccessDeleteAction: RoutesEnum.adminIndexFieldAlias,
      notificationResourceCreateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.index-field-alias.notification.resource.create"),
      notificationResourceDeleteSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.index-field-alias.notification.resource.delete"),
      notificationResourceUpdateSuccessTextToTranslate: MARK_AS_TRANSLATABLE("admin.index-field-alias.notification.resource.update"),
    });
  }

  protected get _urlResource(): string {
    return ApiEnum.indexResourceIndexFieldAliases;
  }

  @Selector()
  static isLoading(state: AdminIndexFieldAliasStateModel): boolean {
    return StoreUtil.isLoadingState(state);
  }

  @Selector()
  static isLoadingWithDependency(state: AdminIndexFieldAliasStateModel): boolean {
    return this.isLoading(state);
  }

  @Selector()
  static currentTitle(state: AdminIndexFieldAliasStateModel): string | undefined {
    if (isNullOrUndefined(state.current)) {
      return undefined;
    }
    return state.current.alias;
  }

  @Selector()
  static isReadyToBeDisplayed(state: AdminIndexFieldAliasStateModel): boolean {
    return this.isReadyToBeDisplayedInCreateMode
      && !isNullOrUndefined(state.current);
  }

  @Selector()
  static isReadyToBeDisplayedInCreateMode(state: AdminIndexFieldAliasStateModel): boolean {
    return true;
  }

  @Action([AdminIndexFieldAliasAction.UpdateSuccess, AdminIndexFieldAliasAction.CreateSuccess, AdminIndexFieldAliasAction.DeleteSuccess])
  refreshSystemProperties(ctx: StateContext<AppSystemPropertyStateModel>): Observable<any> {
    return ctx.dispatch(new AppSystemPropertyAction.GetSystemProperties());
  }
}
