import {AdminLanguageCreateEditDialog} from "@admin/language/components/dialogs/admin-language-create-edit/admin-language-create-edit.dialog";
import {adminLanguageActionNameSpace} from "@admin/language/stores/admin-language.action";
import {AdminLanguageStateModel} from "@admin/language/stores/admin-language.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Injector,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {Language} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {
  AbstractListRoutable,
  DataTableFieldTypeEnum,
  DialogUtil,
  isTrue,
  isTruthyObject,
  OrderEnum,
  RouterExtensionService,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-language-list-routable",
  templateUrl: "../../../../../../../../node_modules/solidify-frontend/lib/components/routables/abstract-list/abstract-list.routable.html",
  styleUrls: ["./admin-language-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminLanguageListRoutable extends AbstractListRoutable<Language, AdminLanguageStateModel> {
  readonly KEY_CREATE_BUTTON: string = LabelTranslateEnum.create;
  readonly KEY_BACK_BUTTON: string | undefined = LabelTranslateEnum.backToAdmin;
  readonly KEY_PARAM_NAME: keyof Language & string = "code";

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtensionService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog,
              protected readonly _injector: Injector) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, StateEnum.admin_language, adminLanguageActionNameSpace, _injector, {}, StateEnum.admin);
  }

  conditionDisplayEditButton(model: Language | undefined): boolean {
    return false;
  }

  conditionDisplayDeleteButton(model: Language | undefined): boolean {
    return true;
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "resId",
        header: LabelTranslateEnum.identifier,
        type: DataTableFieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        width: "100px",
      },
      {
        field: "code",
        header: LabelTranslateEnum.code,
        type: DataTableFieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "creation.when" as any,
        header: LabelTranslateEnum.created,
        type: DataTableFieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "lastUpdate.when" as any,
        header: LabelTranslateEnum.updated,
        type: DataTableFieldTypeEnum.datetime,
        order: OrderEnum.descending,
        isFilterable: true,
        isSortable: true,
      },
    ];
  }

  override create(element: ElementRef): void {
    this.subscribe(DialogUtil.open(this._dialog, AdminLanguageCreateEditDialog, {
      current: undefined,
    }, {
      minWidth: "500px",
    }, isConfirmed => {
      if (isTrue(isConfirmed)) {
        if (isTruthyObject(element)) {
          element.nativeElement.focus();
        }
        this.getAll();
      }
    }, () => {
      if (isTruthyObject(element)) {
        element.nativeElement.focus();
      }
    }));
  }

  override goToEdit(model: Language): void {
  }

  override showDetail(model: Language): void {
  }
}
