import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Injector,
} from "@angular/core";
import {
  FormBuilder,
  Validators,
} from "@angular/forms";
import {PreservationPolicy} from "@models";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {
  AbstractFormPresentational,
  PropertyName,
  SolidifyValidator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-preservation-policy-form",
  templateUrl: "./admin-preservation-policy-form.presentational.html",
  styleUrls: ["./admin-preservation-policy-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminPreservationPolicyFormPresentational extends AbstractFormPresentational<PreservationPolicy> {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  get labelTranslateEnum(): typeof LabelTranslateEnum {
    return LabelTranslateEnum;
  }

  get iconNameEnum(): typeof IconNameEnum {
    return IconNameEnum;
  }

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              protected readonly _injector: Injector,
              private readonly _fb: FormBuilder) {
    super(_changeDetectorRef, _elementRef, _injector);
  }

  protected bindFormTo(preservationPolicies: PreservationPolicy): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: [preservationPolicies.name, [Validators.required, SolidifyValidator]],
      [this.formDefinition.dispositionApproval]: [preservationPolicies.dispositionApproval, [SolidifyValidator]],
      [this.formDefinition.retention]: [preservationPolicies.retention, [Validators.required, SolidifyValidator]],
    });
  }

  protected initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.name]: ["", [Validators.required, SolidifyValidator]],
      [this.formDefinition.dispositionApproval]: [false, [SolidifyValidator]],
      [this.formDefinition.retention]: [undefined, [Validators.required, SolidifyValidator]],
    });
  }

  protected treatmentBeforeSubmit(preservationPolicies: PreservationPolicy): PreservationPolicy {
    return preservationPolicies;
  }

}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() name: string;
  @PropertyName() dispositionApproval: string;
  @PropertyName() retention: string;
}
