import {AdminResearchDomainCreateEditDialog} from "@admin/research-domain/components/dialogs/admin-research-domain-create-edit/admin-research-domain-create-edit.dialog";
import {adminResearchDomainActionNameSpace} from "@admin/research-domain/stores/admin-research-domain.action";
import {AdminResearchDomainStateModel} from "@admin/research-domain/stores/admin-research-domain.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Injector,
  OnInit,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute} from "@angular/router";
import {
  Language,
  ResearchDomain,
} from "@models";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {DataTableComponentHelper} from "@shared/helpers/data-table-component.helper";
import {SharedLanguageAction} from "@shared/stores/language/shared-language.action";
import {SharedLanguageState} from "@shared/stores/language/shared-language.state";
import {Observable} from "rxjs";
import {
  filter,
  take,
} from "rxjs/operators";
import {
  AbstractListRoutable,
  DataTableColumns,
  DataTableFieldTypeEnum,
  DialogUtil,
  isNullOrUndefined,
  isTrue,
  isTruthyObject,
  MARK_AS_TRANSLATABLE,
  MemoizedUtil,
  OrderEnum,
  RouterExtensionService,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-research-domain-list-routable",
  templateUrl: "../../../../../../../../node_modules/solidify-frontend/lib/components/routables/abstract-list/abstract-list.routable.html",
  styleUrls: ["./admin-research-domain-list.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminResearchDomainListRoutable extends AbstractListRoutable<ResearchDomain, AdminResearchDomainStateModel> implements OnInit {
  readonly KEY_CREATE_BUTTON: string = LabelTranslateEnum.create;
  readonly KEY_BACK_BUTTON: string | undefined = LabelTranslateEnum.backToAdmin;
  readonly KEY_PARAM_NAME: keyof ResearchDomain & string = "name";

  languagesObs: Observable<Language[]> = MemoizedUtil.list(this._store, SharedLanguageState);

  constructor(protected readonly _store: Store,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _route: ActivatedRoute,
              protected readonly _routerExt: RouterExtensionService,
              protected readonly _actions$: Actions,
              protected readonly _dialog: MatDialog,
              protected readonly _injector: Injector) {
    super(_store, _changeDetector, _route, _routerExt, _actions$, _dialog, StateEnum.admin_researchDomain, adminResearchDomainActionNameSpace, _injector, {}, StateEnum.admin);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this._store.dispatch(new SharedLanguageAction.GetAll());
    this.subscribe(this.languagesObs.pipe(
        filter(languages => !isNullOrUndefined(languages)),
        take(1),
      ),
      languages => this.addLanguagesColumns(languages));
  }

  conditionDisplayEditButton(model: ResearchDomain | undefined): boolean {
    return true;
  }

  conditionDisplayDeleteButton(model: ResearchDomain | undefined): boolean {
    return true;
  }

  defineColumns(): void {
    this.columns = [
      {
        field: "code",
        header: LabelTranslateEnum.code,
        type: DataTableFieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
        width: "100px",
      },
      {
        field: "name",
        header: LabelTranslateEnum.nameLabel,
        type: DataTableFieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "source" as any,
        header: LabelTranslateEnum.source,
        type: DataTableFieldTypeEnum.string,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "creation.when" as any,
        header: LabelTranslateEnum.created,
        type: DataTableFieldTypeEnum.datetime,
        order: OrderEnum.none,
        isFilterable: true,
        isSortable: true,
      },
      {
        field: "lastUpdate.when" as any,
        header: LabelTranslateEnum.updated,
        type: DataTableFieldTypeEnum.datetime,
        order: OrderEnum.descending,
        isFilterable: true,
        isSortable: true,
      },
    ];
  }

  private addLanguagesColumns(listLanguages: Language[]): void {
    const languagesColumn: DataTableColumns<ResearchDomain>[] = [];
    listLanguages.forEach(language => {
      languagesColumn.push({
        field: language.resId as any,
        header: MARK_AS_TRANSLATABLE(language.resId),
        order: OrderEnum.none,
        component: DataTableComponentHelper.get(DataTableComponentEnum.adminResearchDomainLabel),
      });
    });
    this.columns = [...this.columns, ...languagesColumn];
  }

  override create(element: ElementRef): void {
    this.subscribe(DialogUtil.open(this._dialog, AdminResearchDomainCreateEditDialog, {
        current: undefined,
      },
      {
        minWidth: "500px",
      },
      isConfirmed => {
        if (isTrue(isConfirmed)) {
          if (isTruthyObject(element)) {
            element.nativeElement.focus();
          }
          this.getAll();
        }
      },
      () => {
        if (isTruthyObject(element)) {
          element.nativeElement.focus();
        }
      }));
  }

  override goToEdit(model: ResearchDomain): void {
    this.subscribe(DialogUtil.open(this._dialog, AdminResearchDomainCreateEditDialog, {
        current: model,
      },
      {
        minWidth: "500px",
      },
      isConfirmed => {
        if (isTrue(isConfirmed)) {
          this.getAll();
        }
      }));
  }

  override showDetail(model: ResearchDomain): void {
    this.goToEdit(model);
  }
}
