import {adminUserActionNameSpace} from "@admin/user/stores/admin-user.action";
import {
  AdminUserState,
  AdminUserStateModel,
} from "@admin/user/stores/admin-user.state";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  Injector,
  ViewChild,
} from "@angular/core";
import {
  AccessOrganizationalUnit,
  Person,
  User,
} from "@models";
import {
  Actions,
  Select,
  Store,
} from "@ngxs/store";
import {StateEnum} from "@shared/enums/state.enum";
import {SharedOrganizationalUnitState} from "@shared/stores/organizational-unit/shared-organizational-unit.state";
import {SharedPersonState} from "@shared/stores/person/shared-person.state";
import {sharedUserActionNameSpace} from "@shared/stores/user/shared-user.action";
import {Observable} from "rxjs";
import {
  AbstractCreateRoutable,
  AbstractFormPresentational,
  MemoizedUtil,
  ResourceNameSpace,
} from "solidify-frontend";

@Component({
  selector: "dlcm-admin-user-create-routable",
  templateUrl: "./admin-user-create.routable.html",
  styleUrls: ["./admin-user-create.routable.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AdminUserCreateRoutable extends AbstractCreateRoutable<User, AdminUserStateModel> {
  @Select(AdminUserState.isLoadingWithDependency) isLoadingWithDependencyObs: Observable<boolean>;
  @Select(AdminUserState.isReadyToBeDisplayedInCreateMode) isReadyToBeDisplayedInCreateModeObs: Observable<boolean>;
  listPersonObs: Observable<Person[]> = MemoizedUtil.list(this._store, SharedPersonState);
  listOrgUnitObs: Observable<AccessOrganizationalUnit[]> = MemoizedUtil.list(this._store, SharedOrganizationalUnitState);

  override checkAvailableResourceNameSpace: ResourceNameSpace = sharedUserActionNameSpace;

  @ViewChild("formPresentational")
  readonly formPresentational: AbstractFormPresentational<User>;

  constructor(protected readonly _store: Store,
              protected readonly _actions$: Actions,
              protected readonly _changeDetector: ChangeDetectorRef,
              protected readonly _injector: Injector) {
    super(_store, _actions$, _changeDetector, StateEnum.admin_user, _injector, adminUserActionNameSpace, StateEnum.admin);
  }
}
