import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  OnInit,
} from "@angular/core";
import {
  AbstractControl,
  FormBuilder,
  FormGroup,
  Validators,
} from "@angular/forms";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {DepositDataFile} from "@deposit/models/deposit-data-file.model";
import {DepositDataFileAction} from "@deposit/stores/data-file/deposit-data-file.action";
import {DepositDataFileState} from "@deposit/stores/data-file/deposit-data-file.state";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractDialog} from "@shared/components/dialogs/shared-abstract/shared-abstract.dialog";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {Observable} from "rxjs";
import {
  FormValidationHelper,
  MemoizedUtil,
  PropertyName,
  RegexUtil,
  SolidifyValidator,
  StoreUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-file-move-dialog",
  templateUrl: "./deposit-file-move.dialog.html",
  styleUrls: ["./deposit-file-move.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositFileMoveDialog extends SharedAbstractDialog<DepositFileMoveDialogData, boolean> implements OnInit {
  isLoadingObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, DepositDataFileState);

  form: FormGroup;
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  constructor(private readonly _store: Store,
              private readonly _actions$: Actions,
              protected readonly _dialogRef: MatDialogRef<DepositFileMoveDialog>,
              @Inject(MAT_DIALOG_DATA) public readonly data: DepositFileMoveDialogData,
              private readonly _fb: FormBuilder) {
    super(_dialogRef, data);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.form = this._fb.group({
      [this.formDefinition.subDirectory]: [this.data.depositDataFile.relativeLocation, [Validators.required, Validators.pattern(RegexUtil.subDirectory), SolidifyValidator]],
    });
  }

  onSubmit(): void {
    const action = new DepositDataFileAction.Move(this.data.parentResId, {
      resId: this.data.depositDataFile.resId,
      relativeLocation: this.form.get(this.formDefinition.subDirectory).value,
    });

    this.subscribe(StoreUtil.dispatchActionAndWaitForSubActionCompletion(this._store, this._actions$,
      action,
      DepositDataFileAction.MoveSuccess,
      resultAction => {
        this.submit(true);
      }));
  }

  getFormControl(key: string): AbstractControl {
    return FormValidationHelper.getFormControl(this.form, key);
  }

  get formValidationHelper(): typeof FormValidationHelper {
    return FormValidationHelper;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() subDirectory: string;
}

export interface DepositFileMoveDialogData {
  parentResId: string;
  depositDataFile: DepositDataFile;
}
