import {
  ChangeDetectionStrategy,
  Component,
  Inject,
} from "@angular/core";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {Person} from "@models";
import {SharedAbstractDialog} from "@shared/components/dialogs/shared-abstract/shared-abstract.dialog";
import {isNullOrUndefined} from "solidify-frontend";

@Component({
  selector: "dlcm-deposit-alternative-dialog",
  templateUrl: "./deposit-person-alternative.dialog.html",
  styleUrls: ["./deposit-person-alternative.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class DepositPersonAlternativeDialog extends SharedAbstractDialog<DepositPersonAlternativeDialogData, AlternativeDepositPersonSelected> {
  selectedDepositPerson: AlternativeDepositPersonSelected;

  constructor(protected readonly dialogRef: MatDialogRef<DepositPersonAlternativeDialog>,
              @Inject(MAT_DIALOG_DATA) public data: DepositPersonAlternativeDialogData) {
    super(dialogRef, data);
  }

  select(selected: Person, isNew: boolean): void {
    this.selectedDepositPerson = {
      selected: selected,
      isNew: isNew,
    };
  }

  isSelected(current: Person): boolean {
    if (isNullOrUndefined(this.selectedDepositPerson)) {
      return false;
    }
    return current === this.selectedDepositPerson.selected;
  }
}

export interface DepositPersonAlternativeDialogData {
  newPerson: Person;
  listExistingPerson: Person[];
}

export interface AlternativeDepositPersonSelected {
  selected: Person;
  isNew: boolean;
}
