import {DepositDataFileState} from "@deposit/stores/data-file/deposit-data-file.state";
import {Enums} from "@enums";
import {Store} from "@ngxs/store";
import {
  MappingObjectUtil,
  MemoizedUtil,
} from "solidify-frontend";

export class DataFileUploadHelper {
  private static readonly LIST_PENDING_STATUS: Enums.DataFile.StatusEnum[] = [
    Enums.DataFile.StatusEnum.RECEIVED,
    Enums.DataFile.StatusEnum.CHANGE_RELATIVE_LOCATION,
    Enums.DataFile.StatusEnum.TO_PROCESS,
    Enums.DataFile.StatusEnum.PROCESSED,
    Enums.DataFile.StatusEnum.FILE_FORMAT_IDENTIFIED,
    Enums.DataFile.StatusEnum.FILE_FORMAT_UNKNOWN,
    Enums.DataFile.StatusEnum.VIRUS_CHECKED,
    Enums.DataFile.StatusEnum.CLEANING,
  ];

  static numberFiles(store: Store): number {
    const listCurrentStatus = MemoizedUtil.selectSnapshot(store, DepositDataFileState, state => state.listCurrentStatus);
    let pendingCounter = 0;
    MappingObjectUtil.forEach(listCurrentStatus, (value, key: Enums.DataFile.StatusEnum) => {
      pendingCounter += value;
    });
    return pendingCounter;
  }

  static numberFilesInError(store: Store): number {
    const listCurrentStatus = MemoizedUtil.selectSnapshot(store, DepositDataFileState, state => state.listCurrentStatus);
    return MappingObjectUtil.get(listCurrentStatus, Enums.DataFile.StatusEnum.IN_ERROR) ?? 0;
  }

  static numberFilesIgnored(store: Store): number {
    const listCurrentStatus = MemoizedUtil.selectSnapshot(store, DepositDataFileState, state => state.listCurrentStatus);
    return MappingObjectUtil.get(listCurrentStatus, Enums.DataFile.StatusEnum.IGNORED_FILE) ?? 0;
  }

  static numberFilesExcluded(store: Store): number {
    const listCurrentStatus = MemoizedUtil.selectSnapshot(store, DepositDataFileState, state => state.listCurrentStatus);
    return MappingObjectUtil.get(listCurrentStatus, Enums.DataFile.StatusEnum.EXCLUDED_FILE) ?? 0;
  }

  static numberFilesPending(store: Store): number {
    const listCurrentStatus = MemoizedUtil.selectSnapshot(store, DepositDataFileState, state => state.listCurrentStatus);
    let pendingCounter = 0;
    MappingObjectUtil.forEach(listCurrentStatus, (value, key: Enums.DataFile.StatusEnum) => {
      if (this.LIST_PENDING_STATUS.indexOf(key) !== -1) {
        pendingCounter += value;
      }
    });
    return pendingCounter;
  }
}
