import {Injectable} from "@angular/core";
import {ApiEnum} from "@app/shared/enums/api.enum";
import {DepositDataFile} from "@deposit/models/deposit-data-file.model";
import {depositDataFileStatusHistoryNamespace} from "@deposit/stores/data-file/status-history/deposit-data-file-status-history.action";

import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {StateEnum} from "@shared/enums/state.enum";
import {
  defaultStatusHistoryInitValue,
  StatusHistoryState,
  StatusHistoryStateModel,
} from "@shared/stores/status-history/status-history.state";
import {
  ApiService,
  NotificationService,
} from "solidify-frontend";

export interface DepositDataFileStatusHistoryStateModel extends StatusHistoryStateModel<DepositDataFile> {
}

@Injectable()
@State<DepositDataFileStatusHistoryStateModel>({
  name: StateEnum.deposit_dataFile_statusHistory,
  defaults: {
    ...defaultStatusHistoryInitValue(),
  },
})
export class DepositDataFileStatusHistoryState extends StatusHistoryState<DepositDataFileStatusHistoryStateModel, DepositDataFile> {

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: depositDataFileStatusHistoryNamespace,
    });
  }

  protected get _urlResource(): string {
    return ApiEnum.preIngestDepositDataFile;
  }
}
