import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  OnInit,
} from "@angular/core";
import {
  AbstractControl,
  FormBuilder,
  FormGroup,
  Validators,
} from "@angular/forms";
import {
  MAT_DIALOG_DATA,
  MatDialogRef,
} from "@angular/material/dialog";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {SharedAbstractDialog} from "@shared/components/dialogs/shared-abstract/shared-abstract.dialog";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {SharedNotificationState} from "@shared/stores/notification/shared-notification.state";
import {Observable} from "rxjs";
import {
  FormValidationHelper,
  MARK_AS_TRANSLATABLE,
  MemoizedUtil,
  PropertyName,
  SolidifyValidator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-home-search-help-dialog",
  templateUrl: "./home-search-help.dialog.html",
  styleUrls: ["./home-search-help.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class HomeSearchHelpDialog extends SharedAbstractDialog<HomeSearchHelpDialogData, string> implements OnInit {
  isLoadingObs: Observable<boolean> = MemoizedUtil.isLoading(this._store, SharedNotificationState);

  form: FormGroup;
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  helpInfo: HelpInfo[] = [
    {
      character: "*",
      explain: MARK_AS_TRANSLATABLE("home.search.help.asterisk.explain"),
      example: MARK_AS_TRANSLATABLE("home.search.help.asterisk.example"),
    },
    {
      character: "?",
      explain: MARK_AS_TRANSLATABLE("home.search.help.questionMark.explain"),
      example: MARK_AS_TRANSLATABLE("home.search.help.questionMark.example"),
    },
    {
      character: "AND",
      explain: MARK_AS_TRANSLATABLE("home.search.help.and.explain"),
      example: MARK_AS_TRANSLATABLE("home.search.help.and.example"),
    },
    {
      character: "\"...\"",
      explain: MARK_AS_TRANSLATABLE("home.search.help.quote.explain"),
      example: MARK_AS_TRANSLATABLE("home.search.help.quote.example"),
    },
  ];

  constructor(private readonly _store: Store,
              private readonly _actions$: Actions,
              protected readonly _dialogRef: MatDialogRef<HomeSearchHelpDialog>,
              private readonly _fb: FormBuilder,
              @Inject(MAT_DIALOG_DATA) public readonly data: HomeSearchHelpDialogData) {
    super(_dialogRef, data);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.form = this._fb.group({
      [this.formDefinition.message]: ["", [Validators.required, SolidifyValidator]],
    });
  }

  onSubmit(): void {
    this.submit(this.form.get(this.formDefinition.message).value);
  }

  getFormControl(key: string): AbstractControl {
    return FormValidationHelper.getFormControl(this.form, key);
  }

  get formValidationHelper(): typeof FormValidationHelper {
    return FormValidationHelper;
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() message: string;
}

export interface HomeSearchHelpDialogData {
}

interface HelpInfo {
  character: string;
  explain: string;
  example: string;
}
