import {HttpClient} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {ApiEnum} from "@app/shared/enums/api.enum";
import {HomeArchivePackageAction} from "@home/stores/archive/package/home-archive-package.action";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiResourceNameEnum} from "@shared/enums/api-resource-name.enum";
import {urlSeparator} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {ArchiveMetadataPackages} from "@shared/models/business/archive-metadata-packages.model";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  BaseStateModel,
  BasicState,
  isFalse,
  NotificationService,
  SolidifyHttpErrorResponseModel,
} from "solidify-frontend";

export const defaultHomeArchivePackageValue: () => HomeArchivePackageStateModel = () =>
  ({
    isLoadingCounter: 0,
    package: undefined,
  });

export interface HomeArchivePackageStateModel extends BaseStateModel {
  package: ArchiveMetadataPackages;
}

@Injectable()
@State<HomeArchivePackageStateModel>({
  name: StateEnum.home_archive_package,
  defaults: {
    ...defaultHomeArchivePackageValue(),
  },
  children: [],
})
export class HomeArchivePackageState extends BasicState<HomeArchivePackageStateModel> {
  private readonly _resourceName: ApiResourceNameEnum = ApiResourceNameEnum.PACKAGES;

  protected get _urlResource(): string {
    return ApiEnum.accessPublicMetadata;
  }

  constructor(protected apiService: ApiService,
              protected store: Store,
              protected httpClient: HttpClient,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super();
  }

  @Action(HomeArchivePackageAction.GetById)
  getById(ctx: StateContext<HomeArchivePackageStateModel>, action: HomeArchivePackageAction.GetById): Observable<ArchiveMetadataPackages> {
    const objectToPatch: HomeArchivePackageStateModel | any = {
      isLoadingCounter: ctx.getState().isLoadingCounter + 1,
    };

    if (isFalse(action.keepCurrentContext)) {
      objectToPatch.package = undefined;
    }
    ctx.patchState(objectToPatch);

    return this.apiService.getByIdInPath<ArchiveMetadataPackages>(this._urlResource + urlSeparator + action.resId + urlSeparator + this._resourceName)
      .pipe(
        tap(model => ctx.dispatch(new HomeArchivePackageAction.GetByIdSuccess(action, model))),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new HomeArchivePackageAction.GetByIdFail(action));
          throw error;
        }),
      );
  }

  @Action(HomeArchivePackageAction.GetByIdSuccess)
  getByIdSuccess(ctx: StateContext<HomeArchivePackageStateModel>, action: HomeArchivePackageAction.GetByIdSuccess): void {
    ctx.patchState({
      package: action.model,
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(HomeArchivePackageAction.GetByIdFail)
  getByIdFail(ctx: StateContext<HomeArchivePackageStateModel>, action: HomeArchivePackageAction.GetByIdFail): void {
    ctx.patchState({
      isLoadingCounter: ctx.getState().isLoadingCounter - 1,
    });
  }

  @Action(HomeArchivePackageAction.CleanCurrent)
  cleanCurrent(ctx: StateContext<HomeArchivePackageStateModel>, action: HomeArchivePackageAction.CleanCurrent): void {
    ctx.patchState({
      ...defaultHomeArchivePackageValue(),
    });
  }
}
