import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  ElementRef,
  Injector,
  Output,
} from "@angular/core";
import {FormBuilder} from "@angular/forms";
import {AppUserAction} from "@app/stores/user/app-user.action";
import {AppUserState} from "@app/stores/user/app-user.state";
import {Contributor} from "@models";
import {IconNameEnum} from "@shared/enums/icon-name.enum";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {SecurityService} from "@shared/services/security.service";
import {ResourceLogoNameSpace} from "@shared/stores/resource-logo/resource-logo-namespace.model";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  AbstractFormPresentational,
  BreakpointService,
  isNullOrUndefined,
  ObservableUtil,
  PropertyName,
} from "solidify-frontend";

@Component({
  selector: "dlcm-preservation-space-contributor-form",
  templateUrl: "./preservation-space-contributor-form.presentational.html",
  styleUrls: ["./preservation-space-contributor-form.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class PreservationSpaceContributorFormPresentational extends AbstractFormPresentational<Contributor> {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  userAvatarActionNameSpace: ResourceLogoNameSpace = AppUserAction;
  userState: typeof AppUserState = AppUserState;

  protected readonly _navigatePersonBS: BehaviorSubject<void | undefined> = new BehaviorSubject<void | undefined>(undefined);
  @Output("navigatePerson")
  readonly navigatePersonObs: Observable<void | undefined> = ObservableUtil.asObservable(this._navigatePersonBS);

  get labelTranslateEnum(): typeof LabelTranslateEnum {
    return LabelTranslateEnum;
  }

  get iconNameEnum(): typeof IconNameEnum {
    return IconNameEnum;
  }

  constructor(protected readonly _changeDetectorRef: ChangeDetectorRef,
              protected readonly _elementRef: ElementRef,
              protected readonly _injector: Injector,
              private readonly _fb: FormBuilder,
              public readonly securityService: SecurityService,
              public readonly breakpointService: BreakpointService) {
    super(_changeDetectorRef, _elementRef, _injector);
  }

  protected initNewForm(): void {
    this.form = this._fb.group({
      [this.formDefinition.lastName]: [""],
      [this.formDefinition.firstName]: [""],
      [this.formDefinition.orcid]: [""],
    });
  }

  protected bindFormTo(contributor: Contributor): void {
    this.form = this._fb.group({
      [this.formDefinition.lastName]: [contributor.lastName],
      [this.formDefinition.firstName]: [contributor.firstName],
      [this.formDefinition.orcid]: [contributor.orcid],
    });
  }

  protected treatmentBeforeSubmit(model: Contributor): Contributor {
    return undefined;
  }

  navigateTo($event?: MouseEvent): void {
    this._navigatePersonBS.next();
    if (!isNullOrUndefined($event)) {
      $event.stopPropagation();
    }
  }
}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() lastName: string;
  @PropertyName() firstName: string;
  @PropertyName() orcid: string;
}
