import {
  ChangeDetectionStrategy,
  Component,
  HostBinding,
  Input,
} from "@angular/core";
import {
  DomSanitizer,
  SafeResourceUrl,
} from "@angular/platform-browser";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {UserInfo} from "@models";
import {
  isNotNullNorUndefinedNorWhiteString,
  isNullOrUndefined,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-avatar",
  templateUrl: "./shared-avatar.presentational.html",
  styleUrls: ["./shared-avatar.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedAvatarPresentational extends SharedAbstractPresentational {
  private _userInfo: UserInfo;

  @Input()
  set userInfo(value: UserInfo) {
    this._userInfo = value;
    this.computeInitial();
  }

  get userInfo(): UserInfo {
    return this._userInfo;
  }

  @HostBinding("class.is-photo")
  private _photoUserSanitize: SafeResourceUrl | undefined;

  @Input()
  set photoUser(value: string) {
    if (isNullOrUndefined(value)) {
      this._photoUserSanitize = undefined;
      return;
    }
    this._photoUserSanitize = this.sanitizeUrl(value);
  }

  get photoUserSanitize(): SafeResourceUrl {
    return this._photoUserSanitize;
  }

  @Input()
  @HostBinding("class.is-large")
  large: boolean = false;

  private readonly DEFAULT_INITIAL: string = "";

  initial: string = this.DEFAULT_INITIAL;

  constructor(private readonly _sanitizer: DomSanitizer) {
    super();
  }

  computeInitial(): void {
    if (isNullOrUndefined(this._userInfo)) {
      this.initial = this.DEFAULT_INITIAL;
    } else {
      const firstNameInitial = isNotNullNorUndefinedNorWhiteString(this._userInfo.firstName) ? this._userInfo.firstName.substring(0, 1) : "";
      const lastNameInitial = isNotNullNorUndefinedNorWhiteString(this._userInfo.lastName) ? this._userInfo.lastName.substring(0, 1) : "";
      this.initial = firstNameInitial + lastNameInitial;
    }
  }

  sanitizeUrl(url: string | undefined): undefined | SafeResourceUrl {
    if (isNullOrUndefined(url)) {
      return undefined;
    } else {
      return this._sanitizer.bypassSecurityTrustResourceUrl(url);
    }
  }
}
