import {DragDropModule} from "@angular/cdk/drag-drop";
import {LayoutModule} from "@angular/cdk/layout";
import {TextFieldModule} from "@angular/cdk/text-field";
import {CommonModule} from "@angular/common";
import {NgModule} from "@angular/core";
import {
  FormsModule,
  ReactiveFormsModule,
} from "@angular/forms";
import {MatSpinner} from "@angular/material/progress-spinner";
import {RouterModule} from "@angular/router";
import {MaterialModule} from "@app/material.module";
import {SharedInstitutionState} from "@app/shared/stores/institution/shared-institution.state";
import {SharedLanguageState} from "@app/shared/stores/language/shared-language.state";
import {SharedLicenseState} from "@app/shared/stores/license/shared-license.state";
import {SharedOrganizationalUnitState} from "@app/shared/stores/organizational-unit/shared-organizational-unit.state";
import {SharedPersonState} from "@app/shared/stores/person/shared-person.state";
import {SharedPreservationPolicyState} from "@app/shared/stores/preservation-policy/shared-preservation-policy.state";
import {SharedState} from "@app/shared/stores/shared.state";
import {SharedSubmissionPolicyState} from "@app/shared/stores/submission-policy/shared-submission-policy.state";
import {
  FaIconLibrary,
  FontAwesomeModule,
} from "@fortawesome/angular-fontawesome";
import {fas} from "@fortawesome/free-solid-svg-icons";
import {ArchiveAccessRightService} from "@home/services/archive-access-right.service";
import {FormlyModule} from "@ngx-formly/core";
import {TranslateModule} from "@ngx-translate/core";
import {NgxsModule} from "@ngxs/store";
import {SharedAdditionalInformationPanelContainer} from "@shared/components/containers/shared-additional-information-panel/shared-additional-information-panel.container";
import {SharedArchiveNameContainer} from "@shared/components/containers/shared-archive-name/shared-archive-name.container";
import {SharedAvatarUploadWrapperContainer} from "@shared/components/containers/shared-avatar-upload-wrapper/shared-avatar-upload-wrapper.container";
import {SharedFileAndAipInformationContainer} from "@shared/components/containers/shared-file-and-aip-information/shared-file-and-aip-information.container";
import {SharedImageUploadWrapperContainer} from "@shared/components/containers/shared-image-upload-wrapper/shared-image-upload-wrapper.container";
import {SharedLogoWrapperContainer} from "@shared/components/containers/shared-logo-wrapper/shared-logo-wrapper.container";
import {SharedOrganizationalUnitMemberContainer} from "@shared/components/containers/shared-organizational-unit-member/shared-organizational-unit-member.container";
import {SharedOrganizationalUnitNameContainer} from "@shared/components/containers/shared-organizational-unit-name/shared-organizational-unit-name.container";
import {SharedTableOrgunitContainer} from "@shared/components/containers/shared-table-orgunit/shared-table-orgunit.container";
import {SharedTablePersonOrgunitRoleContainer} from "@shared/components/containers/shared-table-person-orgunit-role/shared-table-person-orgunit-role.container";
import {SharedUserguideSidebarContainer} from "@shared/components/containers/shared-userguide-sidebar/shared-userguide-sidebar.container";
import {SharedInfoExcludedIgnoredFileDialog} from "@shared/components/dialogs/shared-info-excluded-ignored-file/shared-info-excluded-ignored-file.dialog";
import {SharedUploadImageDialog} from "@shared/components/dialogs/shared-upload-image/shared-upload-image.dialog";
import {SharedAccessLevelWithEmbargoPresentational} from "@shared/components/presentationals/shared-access-level-with-embargo/shared-access-level-with-embargo.presentational";
import {SharedAccessLevelPresentational} from "@shared/components/presentationals/shared-access-level/shared-access-level.presentational";
import {SharedAdminResearchDomainLabelPresentational} from "@shared/components/presentationals/shared-admin-research-domain-label/shared-admin-research-domain-label.presentational";
import {SharedAipStatusNamePresentational} from "@shared/components/presentationals/shared-aip-status-name/shared-aip-status-name.presentational";
import {SharedAipStatusOrgunitNamePresentational} from "@shared/components/presentationals/shared-aip-status-orgunit-name/shared-aip-status-orgunit-name.presentational";
import {SharedAipStatusSummaryPresentational} from "@shared/components/presentationals/shared-aip-status-summary/shared-aip-status-summary.presentational";
import {SharedArchivePublicationDatePresentational} from "@shared/components/presentationals/shared-archive-publication-date/shared-archive-publication-date.presentational";
import {SharedArchiveRatingStarPresentational} from "@shared/components/presentationals/shared-archive-rating-star/shared-archive-rating-star.presentational";
import {SharedArchiveTilePresentational} from "@shared/components/presentationals/shared-archive-tile/shared-archive-tile.presentational";
import {SharedAvatarPresentational} from "@shared/components/presentationals/shared-avatar/shared-avatar.presentational";
import {SharedChecksumItemPresentational} from "@shared/components/presentationals/shared-checksum-item/shared-checksum-item.presentational";
import {SharedChecksumPresentational} from "@shared/components/presentationals/shared-checksum/shared-checksum.presentational";
import {SharedComplianceLevelPresentational} from "@shared/components/presentationals/shared-compliance-level/shared-compliance-level.presentational";
import {SharedDataSensitivityInputPresentational} from "@shared/components/presentationals/shared-data-sensitivity-input/shared-data-sensitivity-input.presentational";
import {SharedDataSensitivityPresentational} from "@shared/components/presentationals/shared-data-sensitivity/shared-data-sensitivity.presentational";
import {SharedDatafileQuickStatusPresentational} from "@shared/components/presentationals/shared-datafile-quick-status/shared-datafile-quick-status.presentational";
import {SharedDoiMenuPresentational} from "@shared/components/presentationals/shared-doi-menu/shared-doi-menu.presentational";
import {SharedFolderTreePresentational} from "@shared/components/presentationals/shared-folder-tree/shared-folder-tree.presentational";
import {SharedImageDisplayPresentational} from "@shared/components/presentationals/shared-image-display/shared-image-display.presentational";
import {SharedNotificationFormPresentational} from "@shared/components/presentationals/shared-notification-form/shared-notification-form.presentational";
import {SharedPersonFormPresentational} from "@shared/components/presentationals/shared-person-form/shared-person-form.presentational";
import {SharedPreservationDurationPresentational} from "@shared/components/presentationals/shared-preservation-duration/shared-preservation-duration.presentational";
import {SharedProgressBarPresentational} from "@shared/components/presentationals/shared-progress-bar/shared-progress-bar.presentational";
import {SharedTocPresentational} from "@shared/components/presentationals/shared-toc/shared-toc.presentational";
import {SharedAipCollectionState} from "@shared/features/aip/stores/aip-collection/shared-aip-collection.state";
import {SharedAipCollectionStatusHistoryState} from "@shared/features/aip/stores/aip-collection/status-history/shared-aip-collection-status-history.state";
import {SharedAipDataFileState} from "@shared/features/aip/stores/aip-data-file/shared-aip-data-file.state";
import {SharedAipDataFileStatusHistoryState} from "@shared/features/aip/stores/aip-data-file/status-history/shared-aip-data-file-status-history.state";
import {SharedAipOrganizationalUnitState} from "@shared/features/aip/stores/organizational-unit/shared-aip-organizational-unit.state";
import {SharedAipState} from "@shared/features/aip/stores/shared-aip.state";
import {SharedAipStatusHistoryState} from "@shared/features/aip/stores/status-history/shared-aip-status-history.state";
import {TranslateFacetPipe} from "@shared/pipes/translate-facet.pipe";
import {GoogleAnalyticsService} from "@shared/services/google-analytics.service";
import {OrcidService} from "@shared/services/orcid.service";
import {SecurityService} from "@shared/services/security.service";
import {SharedArchiveAclState} from "@shared/stores/archive-acl/shared-archive-acl.state";
import {SharedArchiveState} from "@shared/stores/archive/shared-archive.state";
import {SharedContributorState} from "@shared/stores/contributor/shared-contributor.state";
import {SharedDepositState} from "@shared/stores/deposit/shared-deposit.state";
import {SharedDisseminationPolicyState} from "@shared/stores/dissemination-policy/shared-dissemination-policy.state";
import {SharedFundingAgencyState} from "@shared/stores/funding-agency/shared-funding-agency.state";
import {SharedIndexFieldAliasState} from "@shared/stores/index-field-alias/shared-index-field-alias.state";
import {SharedMetadataTypeState} from "@shared/stores/metadata-type/shared-metadata-type.state";
import {SharedNotificationState} from "@shared/stores/notification/shared-notification.state";
import {SharedOaiSetState} from "@shared/stores/oai-set/shared-oai-set.state";
import {SharedOrderState} from "@shared/stores/order/shared-order.state";
import {SharedOrganizationalUnitDisseminationPolicyState} from "@shared/stores/organizational-unit/dissemination-policy/shared-organizational.unit-dissemination-policy.state";
import {SharedOrganizationalUnitPreservationPolicyState} from "@shared/stores/organizational-unit/preservation-policy/shared-organizational-unit-preservation-policy.state";
import {SharedOrganizationalUnitSubmissionPolicyState} from "@shared/stores/organizational-unit/submission-policy/shared-organizational-unit-submission-policy.state";
import {SharedResearchDomainState} from "@shared/stores/research-domain/shared-research-domain.state";
import {SharedRoleState} from "@shared/stores/role/shared-role.state";
import {SharedUserState} from "@shared/stores/user/shared-user.state";
import {HighlightModule} from "ngx-highlightjs";
import {ImageCropperModule} from "ngx-image-cropper";
import {TourMatMenuModule} from "ngx-ui-tour-md-menu";
import {SolidifyFrontendModule} from "solidify-frontend";

const routables = [
  SharedFileAndAipInformationContainer,
];
const containers = [
  SharedTableOrgunitContainer,
  SharedOrganizationalUnitNameContainer,
  SharedOrganizationalUnitMemberContainer,
  SharedArchiveNameContainer,
  SharedUserguideSidebarContainer,
  SharedAvatarUploadWrapperContainer,
  SharedImageUploadWrapperContainer,
  SharedTablePersonOrgunitRoleContainer,
  SharedAdditionalInformationPanelContainer,
  SharedLogoWrapperContainer,
];
const dialogs = [
  SharedInfoExcludedIgnoredFileDialog,
  SharedUploadImageDialog,
];
const contents = [];
const presentationals = [
  SharedArchiveRatingStarPresentational,
  SharedAvatarPresentational,
  SharedNotificationFormPresentational,
  SharedTocPresentational,
  SharedComplianceLevelPresentational,
  SharedPersonFormPresentational,
  SharedAipStatusSummaryPresentational,
  SharedAipStatusNamePresentational,
  SharedAipStatusOrgunitNamePresentational,
  SharedDoiMenuPresentational,
  SharedChecksumPresentational,
  SharedChecksumItemPresentational,
  SharedDatafileQuickStatusPresentational,
  SharedAdminResearchDomainLabelPresentational,
  SharedPreservationDurationPresentational,
  SharedAccessLevelPresentational,
  SharedArchivePublicationDatePresentational,
  SharedDataSensitivityPresentational,
  SharedArchiveTilePresentational,
  SharedImageDisplayPresentational,
  SharedFolderTreePresentational,
  SharedDataSensitivityInputPresentational,
  SharedAccessLevelWithEmbargoPresentational,
  SharedProgressBarPresentational,
];
const directives = [];
const pipes = [
  TranslateFacetPipe,
];
const services = [
  SecurityService,
  OrcidService,
  ArchiveAccessRightService,
  GoogleAnalyticsService,
];
const modules = [
  CommonModule,
  FormsModule,
  RouterModule,
  ReactiveFormsModule,
  FormlyModule,
  MaterialModule,
  FontAwesomeModule,
  TranslateModule,
  SolidifyFrontendModule,
  TextFieldModule,
  LayoutModule,
  DragDropModule,
  HighlightModule,
  ImageCropperModule,
  TourMatMenuModule,
];

@NgModule({
  declarations: [
    ...routables,
    ...containers,
    ...dialogs,
    ...contents,
    ...presentationals,
    ...directives,
    ...pipes,
  ],
  imports: [
    ...modules,
    NgxsModule.forFeature([
      SharedState,
      SharedArchiveAclState,
      SharedLicenseState,
      SharedDepositState,
      SharedLanguageState,
      SharedPreservationPolicyState,
      SharedSubmissionPolicyState,
      SharedOrganizationalUnitState,
      SharedDisseminationPolicyState,
      SharedPersonState,
      SharedInstitutionState,
      SharedRoleState,
      SharedOrganizationalUnitSubmissionPolicyState,
      SharedOrganizationalUnitPreservationPolicyState,
      SharedOrganizationalUnitDisseminationPolicyState,
      SharedFundingAgencyState,
      SharedResearchDomainState,
      SharedNotificationState,
      SharedUserState,
      SharedContributorState,
      SharedOrderState,
      SharedOaiSetState,
      SharedMetadataTypeState,
      SharedAipState,
      SharedAipOrganizationalUnitState,
      SharedAipCollectionState,
      SharedAipStatusHistoryState,
      SharedAipCollectionStatusHistoryState,
      SharedAipDataFileState,
      SharedAipDataFileStatusHistoryState,
      SharedArchiveState,
      SharedIndexFieldAliasState,
    ]),
  ],
  exports: [
    ...modules,
    ...routables,
    ...containers,
    ...dialogs,
    ...presentationals,
    ...contents,
    ...directives,
    ...pipes,
  ],
  providers: [
    ...services,
  ],
  entryComponents: [
    ...dialogs,
    ...contents,
    MatSpinner,
  ],
})
export class SharedModule {
  constructor(private library: FaIconLibrary) {
    // Fontawesome library
    library.addIconPacks(fas);
  }
}
