import {PersonRole} from "@admin/models/person-role.model";
import {HttpClient} from "@angular/common/http";
import {Injectable} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {PrivacyPolicyTermsOfUseApprovalDialog} from "@app/components/dialogs/privacy-policy-terms-of-use-approval/privacy-policy-terms-of-use-approval.dialog";
import {
  RoutesEnum,
  urlSeparator,
} from "@app/shared/enums/routes.enum";
import {SessionStorageEnum} from "@app/shared/enums/session-storage.enum";
import {
  appActionNameSpace,
  AppExtendAction,
} from "@app/stores/app.action";
import {AppArchiveAclAction} from "@app/stores/archive-acl/app-archive-acl.action";
import {
  AppArchiveAclState,
  AppArchiveAclStateModel,
} from "@app/stores/archive-acl/app-archive-acl.state";
import {AppAuthorizedOrganizationalUnitAction} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.action";
import {
  AppAuthorizedOrganizationalUnitState,
  AppAuthorizedOrganizationalUnitStateModel,
} from "@app/stores/authorized-organizational-unit/app-authorized-organizational-unit.state";
import {
  AppBannerState,
  AppBannerStateModel,
} from "@app/stores/banner/app-banner.state";
import {AppCarouselAction} from "@app/stores/carousel/app-carousel.action";
import {
  AppCarouselState,
  AppCarouselStateModel,
} from "@app/stores/carousel/app-carousel.state";
import {AppCartAction} from "@app/stores/cart/app-cart.action";
import {AppCartState} from "@app/stores/cart/app-cart.state";
import {AppCartOrderAction} from "@app/stores/cart/order/app-cart-order.action";
import {AppCartOrderState} from "@app/stores/cart/order/app-cart-order.state";
import {AppNotificationInboxAction} from "@app/stores/notification-inbox/app-notification-inbox.action";
import {AppNotificationInboxState} from "@app/stores/notification-inbox/app-notification-inbox.state";
import {
  AppOrganizationalUnitPersonRoleState,
  AppOrganizationalUnitPersonRoleStateModel,
} from "@app/stores/organizational-unit-person-role/app-organizational-unit-person-role.state";
import {
  AppPersonState,
  AppPersonStateModel,
} from "@app/stores/person/app-person.state";
import {AppSystemPropertyAction} from "@app/stores/system-property/app-system-property.action";
import {
  AppSystemPropertyState,
  AppSystemPropertyStateModel,
} from "@app/stores/system-property/app-system-property.state";
import {AppTocAction} from "@app/stores/toc/app-toc.action";
import {
  AppTocState,
  AppTocStateModel,
} from "@app/stores/toc/app-toc.state";
import {AppUserAction} from "@app/stores/user/app-user.action";
import {
  AppUserState,
  defaultAppUserStateModel,
} from "@app/stores/user/app-user.state";
import {Enums} from "@enums";
import {environment} from "@environments/environment";
import {
  ApplicationRole,
  NotificationDlcm,
  OrganizationalUnit,
  Person,
  Role,
  User,
} from "@models";
import {TranslateService} from "@ngx-translate/core";
import {Navigate} from "@ngxs/router-plugin";
import {
  Action,
  Actions,
  Selector,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {ApiEnum} from "@shared/enums/api.enum";
import {LabelTranslateEnum} from "@shared/enums/label-translate.enum";
import {LocalStorageEnum} from "@shared/enums/local-storage.enum";
import {ShortDoi} from "@shared/models/short-doi.model";
import {Token} from "@shared/models/token.model";
import {Observable} from "rxjs";
import {
  catchError,
  tap,
} from "rxjs/operators";
import {
  ApiService,
  ClipboardUtil,
  defaultBaseStateInitValue,
  DialogUtil,
  isNullOrUndefined,
  isTrue,
  LocalStorageHelper,
  MappingObjectUtil,
  MARK_AS_TRANSLATABLE,
  MemoizedUtil,
  NotificationService,
  OAuth2Service,
  ofSolidifyActionCompleted,
  OverrideDefaultAction,
  PollingHelper,
  QueryParameters,
  SessionStorageHelper,
  SOLIDIFY_CONSTANTS,
  SolidifyAppAction,
  SolidifyAppState,
  SolidifyAppStateModel,
  SolidifyAppUserStateModel,
  SolidifyHttpErrorResponseModel,
  StoreUtil,
} from "solidify-frontend";
import {ApiActionNameEnum} from "../shared/enums/api-action-name.enum";
import {StateEnum} from "../shared/enums/state.enum";

export interface AppStateModel extends SolidifyAppStateModel {
  userRoles: Enums.UserApplicationRole.UserApplicationRoleEnum[];
  application_person: AppPersonStateModel | undefined;
  application_authorizedOrganizationalUnit: AppAuthorizedOrganizationalUnitStateModel | undefined;
  application_organizationalUnit_personRole: AppOrganizationalUnitPersonRoleStateModel | undefined;
  application_toc: AppTocStateModel | undefined;
  application_systemProperty: AppSystemPropertyStateModel;
  displayCookieConsent: boolean;
  [StateEnum.application_banner]: AppBannerStateModel | undefined;
  [StateEnum.application_carousel]: AppCarouselStateModel | undefined;
  [StateEnum.application_archiveAcl]: AppArchiveAclStateModel | undefined;
}

@Injectable()
@State<AppStateModel>({
  name: StateEnum.application,
  defaults: {
    ...defaultBaseStateInitValue(),
    isTouchInterface: false,
    isApplicationInitialized: false,
    isServerOffline: false,
    isLoggedIn: false,
    isInTourMode: false,
    ignorePreventLeavePopup: false,
    isOpenedSidebarUserGuide: false,
    userRoles: [Enums.UserApplicationRole.UserApplicationRoleEnum.guest],
    appLanguage: undefined,
    theme: environment.theme,
    preventExit: false,
    preventExitReasonToTranslate: undefined,
    application_user: undefined,
    application_person: undefined,
    application_authorizedOrganizationalUnit: undefined,
    application_organizationalUnit_personRole: undefined,
    application_toc: undefined,
    token: undefined,
    application_systemProperty: undefined,
    displayCookieConsent: false,
    [StateEnum.application_banner]: undefined,
    [StateEnum.application_carousel]: undefined,
    [StateEnum.application_archiveAcl]: undefined,
  },
  children: [
    AppUserState,
    AppPersonState,
    AppAuthorizedOrganizationalUnitState,
    AppOrganizationalUnitPersonRoleState,
    AppCartState,
    AppTocState,
    AppNotificationInboxState,
    AppSystemPropertyState,
    AppBannerState,
    AppCarouselState,
    AppArchiveAclState,
  ],
})
export class AppState extends SolidifyAppState<AppStateModel> {
  constructor(protected readonly _store: Store,
              protected readonly _translate: TranslateService,
              protected readonly _oauthService: OAuth2Service,
              protected readonly _actions$: Actions,
              protected readonly _apiService: ApiService,
              protected readonly _httpClient: HttpClient,
              protected readonly _notificationService: NotificationService,
              public _dialog: MatDialog) {
    super(_store,
      _translate,
      _oauthService,
      _actions$,
      _apiService,
      _httpClient,
      _notificationService,
      {
        nameSpace: appActionNameSpace,
        baseAppSate: AppState,
        routeServerOffline: RoutesEnum.serverOffline,
        urlRevokeToken: () => ApiEnum.adminUsers + SOLIDIFY_CONSTANTS.URL_SEPARATOR + ApiActionNameEnum.REVOKE_MY_TOKENS,
        urlAdminModule: () => ApiEnum.preservationPlanningModule,
        initApplicationParallel: [
          {action: new AppExtendAction.LoadCart()}, // TODO See if should not be differed
          {action: new AppExtendAction.StartPollingOrder()},
          {action: new AppExtendAction.StartPollingNotification()},
          {action: new AppTocAction.GetAllDocumentation()},
          {
            action: new AppSystemPropertyAction.GetSystemProperties(),
            subActionCompletions: [
              _actions$.pipe(ofSolidifyActionCompleted(AppSystemPropertyAction.GetSystemPropertiesSuccess)),
              _actions$.pipe(ofSolidifyActionCompleted(AppSystemPropertyAction.GetSystemPropertiesFail)),
            ],
          },
        ],
        forceLogin: false,
        loadModule: true,
      },
      environment);
  }

  @Selector()
  static listAuthorizedOrganizationalUnit(state: AppStateModel): OrganizationalUnit[] {
    return state.application_authorizedOrganizationalUnit.list;
  }

  @Selector()
  static listAuthorizedOrganizationalUnitId(state: AppStateModel): string[] {
    const list = this.listAuthorizedOrganizationalUnit(state);
    if (isNullOrUndefined(list)) {
      return [];
    }
    return list.map(o => o.resId);
  }

  @Selector()
  static currentPerson(state: AppStateModel): Person {
    return state.application_person.current;
  }

  @Selector()
  static currentUser(state: AppStateModel): User {
    return state.application_user.current;
  }

  @Selector()
  static currentUserApplicationRole(state: AppStateModel): ApplicationRole {
    const currentPersonRole = this.currentUser(state);
    if (isNullOrUndefined(currentPersonRole)) {
      return undefined;
    }
    return currentPersonRole.applicationRole;
  }

  @Selector()
  static currentUserApplicationRoleResId(state: AppStateModel): string | undefined {
    const applicationRole = this.currentUserApplicationRole(state);
    if (isNullOrUndefined(applicationRole)) {
      return undefined;
    }
    return applicationRole.resId;
  }

  @Selector()
  static currentOrgUnitPerson(state: AppStateModel): PersonRole | undefined {
    if (isNullOrUndefined(state.application_organizationalUnit_personRole.current)) {
      return undefined;
    }
    return state.application_organizationalUnit_personRole.current;
  }

  @Selector()
  static currentOrgUnitPersonRole(state: AppStateModel): Role[] | undefined {
    const currentPerson = this.currentOrgUnitPerson(state);
    if (isNullOrUndefined(currentPerson)) {
      return undefined;
    }
    return currentPerson.roles;
  }

  @Selector()
  static currentOrgUnitPersonRoleResId(state: AppStateModel): string[] | undefined {
    const currentPersonRole = this.currentOrgUnitPersonRole(state);
    if (isNullOrUndefined(currentPersonRole)) {
      return undefined;
    }
    return currentPersonRole.map(r => r.resId);
  }

  @OverrideDefaultAction()
  @Action(SolidifyAppAction.ChangeAppLanguageSuccess)
  override changeAppLanguageSuccess(ctx: StateContext<AppStateModel>, action: SolidifyAppAction.ChangeAppLanguageSuccess): void {
    super.changeAppLanguageSuccess(ctx, action);
    ctx.dispatch(new AppCarouselAction.GetCarousel());
  }

  @OverrideDefaultAction()
  @Action(SolidifyAppAction.ChangeAppTheme)
  override changeAppTheme(ctx: StateContext<AppStateModel>, action: SolidifyAppAction.ChangeAppTheme): void {
    super.changeAppTheme(ctx, action);
    ctx.dispatch(new AppCarouselAction.GetCarousel());
  }

  @Action(AppExtendAction.LoadCart)
  loadCart(ctx: StateContext<AppStateModel>, action: AppExtendAction.LoadCart): void {
    ctx.dispatch(new AppCartAction.GetAllCart());
  }

  @Action(AppExtendAction.StartPollingOrder)
  startPollingOrder(ctx: StateContext<AppStateModel>, action: AppExtendAction.StartPollingOrder): void {
    this.subscribe(PollingHelper.startPollingObs({
      initialIntervalRefreshInSecond: environment.refreshOrderAvailableIntervalInSecond,
      incrementInterval: true,
      resetIntervalWhenUserMouseEvent: true,
      maximumIntervalRefreshInSecond: environment.refreshOrderAvailableIntervalInSecond * 10,
      filter: () => SessionStorageHelper.getListItem(SessionStorageEnum.orderPending).length > 0,
      actionToDo: () => {
        const listOrderPending = SessionStorageHelper.getListItem(SessionStorageEnum.orderPending);
        const orderList = MemoizedUtil.listSnapshot(this._store, AppCartOrderState);
        listOrderPending.forEach(orderId => {
          let order = undefined;
          if (!isNullOrUndefined(orderList)) {
            order = orderList.find(o => o.resId === orderId);
          }
          if (!isNullOrUndefined(order) && (order.status === Enums.Order.StatusEnum.READY || order.status === Enums.Order.StatusEnum.IN_ERROR)) {
            SessionStorageHelper.removeItemInList(SessionStorageEnum.orderPending, orderId);
            SessionStorageHelper.addItemInList(SessionStorageEnum.newOrderAvailable, orderId);
            const actionNotification = {
              text: LabelTranslateEnum.see,
              callback: () => this._store.dispatch(new Navigate([RoutesEnum.orderMyOrderDetail, orderId])),
            };
            if (order.status === Enums.Order.StatusEnum.READY) {
              this._notificationService.showSuccess(MARK_AS_TRANSLATABLE("app.notification.order.ready"), undefined, actionNotification);
            } else {
              this._notificationService.showError(MARK_AS_TRANSLATABLE("app.notification.order.inError"), undefined, actionNotification);
            }
          } else {
            ctx.dispatch(new AppCartOrderAction.AddInListById(orderId, true, true));
          }
        });
      },
    }));
  }

  @Action(AppExtendAction.StartPollingNotification)
  startPollingNotification(ctx: StateContext<AppStateModel>, action: AppExtendAction.StartPollingNotification): void {
    if (ctx.getState().isLoggedIn) {
      this._store.dispatch(new AppExtendAction.UpdateNotificationInbox());
    }

    this.subscribe(PollingHelper.startPollingObs({
      initialIntervalRefreshInSecond: environment.refreshNotificationInboxAvailableIntervalInSecond,
      incrementInterval: true,
      resetIntervalWhenUserMouseEvent: true,
      maximumIntervalRefreshInSecond: environment.refreshNotificationInboxAvailableIntervalInSecond * 10,
      filter: () => ctx.getState().isLoggedIn,
      actionToDo: () => {
        this._store.dispatch(new AppExtendAction.UpdateNotificationInbox());
      },
    }));
  }

  @Action(AppExtendAction.UpdateNotificationInbox)
  updateNotificationInbox(ctx: StateContext<AppStateModel>, action: AppExtendAction.UpdateNotificationInbox): Observable<AppNotificationInboxAction.GetAllSuccess> {
    const queryParameters = new QueryParameters(environment.maximalPageSizeToRetrievePaginationInfo);
    const searchItems = queryParameters.search.searchItems;
    const NOTIFICATION_STATUS: keyof NotificationDlcm = "notificationStatus";
    MappingObjectUtil.set(searchItems, NOTIFICATION_STATUS, Enums.Notification.StatusEnum.PENDING);

    return StoreUtil.dispatchActionAndWaitForSubActionCompletion(this._store, this._actions$,
      new AppNotificationInboxAction.GetAll(queryParameters),
      AppNotificationInboxAction.GetAllSuccess,
      resultAction => {
        const listNewNotificationInboxPending = resultAction.list._data.map(newNotification => newNotification.resId);
        const listOldNotificationInboxPending = SessionStorageHelper.getListItem(SessionStorageEnum.notificationInboxPending);
        let counterNewNotification = 0;
        listNewNotificationInboxPending.forEach(newNotificationId => {
          if (!listOldNotificationInboxPending.includes(newNotificationId)) {
            counterNewNotification++;
          }
        });
        if (counterNewNotification > 0) {
          const actionNotification = {
            text: LabelTranslateEnum.see,
            callback: () => this._store.dispatch(new Navigate([RoutesEnum.preservationSpaceNotificationInbox])),
          };
          this._notificationService.showSuccess(MARK_AS_TRANSLATABLE("app.notification.notificationInbox.newIncoming"), {count: counterNewNotification}, actionNotification);
        }
        SessionStorageHelper.initItemInList(SessionStorageEnum.notificationInboxPending, listNewNotificationInboxPending);
      });
  }

  override loginSuccess(ctx: StateContext<AppStateModel>): Token {
    const tokenDecoded = super.loginSuccess(ctx) as Token;

    ctx.patchState({
      isLoggedIn: true,
      userRoles: tokenDecoded.authorities,
      token: tokenDecoded,
    });

    this.subscribe(StoreUtil.dispatchParallelActionAndWaitForSubActionsCompletion(ctx, [
      {
        action: new AppUserAction.GetCurrentUser(),
        subActionCompletions: [
          this._actions$.pipe(ofSolidifyActionCompleted(AppUserAction.GetCurrentUserSuccess)),
          this._actions$.pipe(ofSolidifyActionCompleted(AppUserAction.GetCurrentUserFail)),
        ],
      },
      {
        action: new AppAuthorizedOrganizationalUnitAction.GetAll(),
        subActionCompletions: [
          this._actions$.pipe(ofSolidifyActionCompleted(AppAuthorizedOrganizationalUnitAction.GetAllSuccess)),
          this._actions$.pipe(ofSolidifyActionCompleted(AppAuthorizedOrganizationalUnitAction.GetAllFail)),
        ],
      },
      {
        action: new AppArchiveAclAction.GetAll(),
        subActionCompletions: [
          this._actions$.pipe(ofSolidifyActionCompleted(AppArchiveAclAction.GetAllSuccess)),
          this._actions$.pipe(ofSolidifyActionCompleted(AppArchiveAclAction.GetAllFail)),
        ],
      },
    ]));

    return tokenDecoded;
  }

  @Action(AppExtendAction.InitiateNewLogin)
  initiateNewLogin(ctx: StateContext<AppStateModel>, action: AppExtendAction.InitiateNewLogin): void {
    const privacyPolicyAndTermsOfUseAccepted = LocalStorageHelper.getItem(LocalStorageEnum.privacyPolicyAndTermsOfUseAccepted);
    if (isTrue(environment.displayPrivacyPolicyAndTermsOfUseApprovalDialog) && privacyPolicyAndTermsOfUseAccepted !== "true") {
      this.subscribe(DialogUtil.open(this._dialog, PrivacyPolicyTermsOfUseApprovalDialog, {}, {
        takeOne: true,
      }, result => {
        if (isTrue(result)) {
          LocalStorageHelper.setItem(LocalStorageEnum.privacyPolicyAndTermsOfUseAccepted, "true");
          this._oauthService.initAuthorizationCodeFlow();
        }
      }));
    } else {
      this._oauthService.initAuthorizationCodeFlow();
    }
  }

  override logout(ctx: StateContext<AppStateModel>): Observable<any> {
    const result = super.logout(ctx);
    if (isNullOrUndefined(result)) {
      ctx.dispatch(new Navigate([environment.routeHomePage]));
      return;
    }
    ctx.patchState({
      ignorePreventLeavePopup: true,
      isLoggedIn: false,
      userRoles: [Enums.UserApplicationRole.UserApplicationRoleEnum.guest],
      token: undefined,
      application_user: {...defaultAppUserStateModel()} as SolidifyAppUserStateModel,
    });
    return result;
  }

  @Action(AppExtendAction.GetShortDoi)
  getShortDoi(ctx: StateContext<AppStateModel>, action: AppExtendAction.GetShortDoi): Observable<any> {
    return this._httpClient.get<ShortDoi>("/api/short-doi" + urlSeparator + action.doiWrapper.longDoi, {})
      .pipe(
        tap(result => {
          ctx.dispatch(new AppExtendAction.GetShortDoiSuccess(action, result.ShortDOI));
        }),
        catchError((error: SolidifyHttpErrorResponseModel) => {
          ctx.dispatch(new AppExtendAction.GetShortDoiFail(action));
          throw error;
        }),
      );
  }

  @Action(AppExtendAction.GetShortDoiSuccess)
  getShortDoiSuccess(ctx: StateContext<AppStateModel>, action: AppExtendAction.GetShortDoiSuccess): void {
    let stringToCopy = action.shortDoi;
    let notificationToTranslate = MARK_AS_TRANSLATABLE("doi.notification.shortDoiCopyToClipboard.success");
    if (action.parent.doiWrapper.isFullUrl) {
      stringToCopy = environment.doiLink + stringToCopy;
      notificationToTranslate = MARK_AS_TRANSLATABLE("doi.notification.urlShortDoiCopyToClipboard.success");
    }
    ClipboardUtil.copyStringToClipboard(stringToCopy);
    this._notificationService.showInformation(notificationToTranslate);
  }

  @Action(AppExtendAction.GetShortDoiFail)
  getShortDoiFail(ctx: StateContext<AppStateModel>, action: AppExtendAction.GetShortDoiFail): void {
    let notificationToTranslate = MARK_AS_TRANSLATABLE("doi.notification.shortDoiCopyToClipboard.fail");
    if (action.parent.doiWrapper.isFullUrl) {
      notificationToTranslate = MARK_AS_TRANSLATABLE("doi.notification.urlShortDoiCopyToClipboard.fail");
    }
    this._notificationService.showWarning(notificationToTranslate);
  }

  @Action(AppExtendAction.DisplayCookieConsent)
  displayCookieConsent(ctx: StateContext<AppStateModel>, action: AppExtendAction.DisplayCookieConsent): void {
    ctx.patchState({
      displayCookieConsent: action.display,
    });
  }
}
