import {Injectable} from "@angular/core";
import {appArchiveAclActionNameSpace} from "@app/stores/archive-acl/app-archive-acl.action";
import {environment} from "@environments/environment";
import {ArchiveACL} from "@models";
import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {ApiEnum} from "@shared/enums/api.enum";
import {urlSeparator} from "@shared/enums/routes.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {
  ApiService,
  defaultResourceStateInitValue,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
} from "solidify-frontend";
import {ApiActionNameEnum} from "../../shared/enums/api-action-name.enum";

export interface AppArchiveAclStateModel extends ResourceStateModel<ArchiveACL> {
}

@Injectable()
@State<AppArchiveAclStateModel>({
  name: StateEnum.application_archiveAcl,
  defaults: {
    ...defaultResourceStateInitValue(),
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
  children: [],
})
export class AppArchiveAclState extends ResourceState<AppArchiveAclStateModel, ArchiveACL> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: appArchiveAclActionNameSpace,
    });
  }

  protected get _urlResource(): string {
    return ApiEnum.adminArchiveAcl + urlSeparator + ApiActionNameEnum.GET_MY_ACLS;
  }
}
