import {Injectable} from "@angular/core";
import {
  AppCartDipDataFileAction,
  appCartDipDataFileActionNameSpace,
} from "@app/stores/cart/dip/data-file/app-cart-dip-data-file.action";
import {
  Action,
  Actions,
  State,
  StateContext,
  Store,
} from "@ngxs/store";
import {DipDataFile} from "@preservation-planning/dip/models/dip-data-file.model";
import {ApiResourceNameEnum} from "@shared/enums/api-resource-name.enum";
import {ApiEnum} from "@shared/enums/api.enum";
import {StateEnum} from "@shared/enums/state.enum";
import {
  ApiService,
  CompositionState,
  CompositionStateModel,
  defaultCompositionStateInitValue,
  DownloadService,
  NotificationService,
} from "solidify-frontend";
import {ApiActionNameEnum} from "../../../../shared/enums/api-action-name.enum";

export const defaultAppCartDipDataFileValue: () => AppCartDipDataFileStateModel = () =>
  ({
    ...defaultCompositionStateInitValue(),
  });

export interface AppCartDipDataFileStateModel extends CompositionStateModel<DipDataFile> {
}

@Injectable()
@State<AppCartDipDataFileStateModel>({
  name: StateEnum.application_cart_dip_dataFile,
  defaults: {
    ...defaultAppCartDipDataFileValue(),
  },
  children: [],
})
export class AppCartDipDataFileState extends CompositionState<AppCartDipDataFileStateModel, DipDataFile> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions,
              private downloadService: DownloadService) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: appCartDipDataFileActionNameSpace,
      resourceName: ApiResourceNameEnum.DATAFILE,
    });
  }

  protected get _urlResource(): string {
    return ApiEnum.accessDip;
  }

  @Action(AppCartDipDataFileAction.Download)
  download(ctx: StateContext<AppCartDipDataFileStateModel>, action: AppCartDipDataFileAction.Download): void {
    const url = `${this._urlResource}/${action.parentId}/${this._resourceName}/${action.dataFile.resId}/${ApiActionNameEnum.DL}`;
    this.downloadService.download(url, action.dataFile.fileName, action.dataFile.fileSize);
  }
}
