// This file should contains all the defaults settings shared in all environment files
// If you want to override properties for specific environment, do it in these files

import {ErrorBackendKeyEnum} from "@app/shared/enums/error-backend-key.enum";
import {ThemeEnum} from "@app/shared/enums/theme.enum";
import {Enums} from "@enums";
import {
  AppRoutesEnum,
  RoutesEnum,
} from "@shared/enums/routes.enum";
import {ViewModeTableEnum} from "@shared/enums/view-mode-table.enum";
import {defaultSolidifyEnvironment} from "solidify-frontend";
import {DlcmEnvironment} from "./environment.defaults.model";

export const defaultEnvironment: DlcmEnvironment = {
  ...defaultSolidifyEnvironment,
  scope: "dlcm",
  appTitle: "DLCM - Portal",
  appDescription: "Platform for archiving research data",
  listTagsComponentsWhereClickAllowToEnterInEditMode: [
    "dlcm-shared-table-person-orgunit-role-container",
    "dlcm-shared-table-orgunit-container",
  ],
  defaultLanguage: Enums.Language.LanguageEnum.en,
  appLanguages: [Enums.Language.LanguageEnum.en, Enums.Language.LanguageEnum.fr],
  appLanguagesTranslate: Enums.Language.LanguageEnumTranslate,
  appThemes: [ThemeEnum.dlcm, ThemeEnum.yareta, ThemeEnum.olos],
  appThemesTranslate: [
    {key: ThemeEnum.dlcm, value: "DLCM"},
    {key: ThemeEnum.yareta, value: "Yareta"},
    {key: ThemeEnum.olos, value: "Olos"},
  ],
  theme: ThemeEnum.dlcm,
  routeHomePage: RoutesEnum.homePage,
  routeSegmentEdit: AppRoutesEnum.edit,
  googleAnalyticsLearnMorePage: undefined,
  displayPrivacyPolicyAndTermsOfUseApprovalDialog: false,
  privacyPolicyLink: "",
  termsOfUseLink: "",

  // Base URL For Module Applications (override by AppAction.LoadModules)
  admin: "http://localhost:16105/dlcm/admin",

  // Documentation
  documentationTocApisGuide: "DLCM-APIs-toc.html",
  documentationTocIntegrationGuide: "DLCM-IntegrationGuide-toc.html",
  documentationTocToolsGuide: "DLCM-ToolsGuide-toc.html",
  documentationTocUserGuidePath: undefined,
  documentationTocUserGuide: undefined,

  // Orcid
  orcidQueryParam: "orcid",
  orcidAuthorizeUrl: "https://orcid.org/oauth/authorize",
  orcidUrl: "http://orcid.org",

  // Access request
  accessRequestRoute: "accessRequest",

  institutionUrl: "https://www.unige.ch/",

  // === Solidify Upload Environment ===
  errorUploadDuplicateFile: ErrorBackendKeyEnum.UPLOAD_DUPLICATE_DATA_FILES,

  // Home
  defaultHomeViewModeTableEnum: ViewModeTableEnum.tiles,
  defaultPageSizeHomePage: 10,
  defaultHomeOrgUnitSize: 1000,
  carouselUrl: undefined,
  frequencyChangeCarouselTileInSecond: 5,
  twitterAccount: "dlcm_ch",
  twitterTweetToDisplay: 4,

  urlNationalArchivePronom: "https://www.nationalarchives.gov.uk/PRONOM/",

  doiLink: "https://doi.org/",
  oaiProviderUrlEnd: "/oai-provider/oai?verb=Identify&smartView=dlcm_oai2.xsl",

  // Polling
  refreshTabStatusCounterIntervalInSecond: 60,
  refreshOrderAvailableIntervalInSecond: 5,
  refreshNotificationInboxAvailableIntervalInSecond: 60,
  refreshDepositSubmittedIntervalInSecond: 2,
  refreshSipProcessIntervalInSecond: 2,

  // Errors
  httpErrorKeyToSkipInErrorHandler: [
    ErrorBackendKeyEnum.UPLOAD_DUPLICATE_DATA_FILES,
  ],

  archivalStorageName: [
    "File",
    "S2",
  ],
  defaultStorageIndex: 0,
  fileSizeLimitForChecksumGenerationInMegabytes: 20,

  listFacetUseOperatorAnd: [Enums.Facet.Name.CREATOR_FACET],
};

/*
 * For easier debugging in development mode, you can import the following file
 * to ignore zone related error stack frames such as `zone.run`, `zoneDelegate.invokeTask`.
 *
 * This import should be commented out in production mode because it will have a negative impact
 * on performance if an error is thrown.
 */
// import 'zone.js/plugins/zone-error';  // Included with Angular CLI.
